#!/usr/bin/env python

# Copyright 2008 Vojtech Kovar

# This file is part of SET.
#
# SET is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# SET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with SET.  If not, see <http://www.gnu.org/licenses/>.


# the top-level code of the SET parser

import os, sys, getopt
import locale

import settings
from grammar import Grammar
from parser import Parser
from segment import Segment
from vertical_processor import VerticalProcessor
from utils import log, usage

locale.setlocale(locale.LC_ALL, '')

version = '0.4'

graph_out = False
dep_out = False
phr_out = False
laa_out = False
collx_out = False
pos_tags = False
use_collx = False
phrases = False
print_tree = False
marx = False # export phrases for Marek Grac's ontology building

set_path = os.path.dirname(sys.argv[0])
grammar = os.path.join(set_path, 'grammar.set') # might be variable in the future
lw_collx = os.path.join(set_path, 'data', 'lemma-word')
ll_collx = os.path.join(set_path, 'data', 'lemma-lemma')
lt_collx = os.path.join(set_path, 'data', 'lemma-tag')

# parse the command line
try: options, args = getopt.getopt(sys.argv[1:], 'gdpct',
		['version', 'laa', 'postags', 'cout', 'phrases', 'marx'])
except: usage(sys.argv[0]); exit(1)

for option, value in options:
	if option == '-g': graph_out = True
	if option == '-t': print_tree = True
	if option == '-d': dep_out = True
	if option == '-p': phr_out = True
	if option == '-c': use_collx = True
	if option == '--postags': pos_tags = True
	if option == '--cout': collx_out = True
	if option == '--phrases': phrases = True
	if option == '--laa': laa_out = True
	if option == '--marx': marx = True
	if option == '--version': print 'SET version %s' % version

if (not print_tree) and (dep_out or phr_out or collx_out or phrases or 
																				laa_out or marx):
	print_tree = False
else:
	print_tree = True

if not args: usage(sys.argv[0]); exit(1)
filename = args[0]

if dep_out and (laa_out or phr_out):
	log('Incompatible options: -d not compatible with -p and --laa');
	exit(3)
if collx_out and (dep_out or laa_out or phr_out):
	log('Incompatible options: --cout not compatible with -d, -p and --laa')
	exit(3)

# open the input file
try:
	file = open(filename)
except Exception, detail:
	log('Cannot open the file %s : %s' % (filename, str(detail)) )
	exit(2)

# get separate sentences from vertical processor
vert_processor = VerticalProcessor(file)
lines, end_tag = vert_processor.get_sentence()
while lines:

	# init structures
	g = Grammar(grammar)
	s = Segment(lines, pos_tags=pos_tags) # load file with sentence
	if use_collx:
		p = Parser(g, lw_collx_root=lw_collx, ll_collx_root=ll_collx,
																			lt_collx_root=lt_collx)
	else:
		p = Parser(g)

	# parsing and output
	p.parse(s)

	if dep_out: s.print_dep_tree(root=False)
	if laa_out: s.print_laa_tree()
	if phr_out: s.print_phr_tree()
	if collx_out: s.print_collx_out()
	if print_tree: s.print_tree()
	if phrases: s.print_phrases()
	if marx: s.print_marx_phrases()

	if graph_out:
		#from tree_view import display_tree
		from TreeViewer.TreeViewer import initGUI
		if dep_out: initGUI([s.get_dep_tree(root=True)])
		elif phr_out or laa_out: initGUI([s.get_phr_tree()])
		else: initGUI([s.get_tree()])

	if end_tag and not settings.IGNORETAGS: sys.stdout.write(end_tag)
	lines, end_tag = vert_processor.get_sentence()

file.close()



